@@ -9,7 +9,7 @@
 #ifndef __GIVARO_random_integer_iterator_H
 #define __GIVARO_random_integer_iterator_H
 
-#include "givaro/givinteger.h"
+#include "gmp++/gmp++.h"
 
 namespace std {
     template <bool B>
@@ -31,6 +31,8 @@ namespace Givaro
      * @tparam _Exact_Size if \c true, then random integers have
      * exactly the number of required bits, if \c false, they have
      * less than the required number of bits
+     * WARNING, the seed is *** global ***
+     * WARNING: i.e. sets the seed for all Givaro Integer 
      */
     template<bool _Unsigned=true, bool _Exact_Size=false>
     class RandomIntegerIterator {
@@ -39,18 +41,14 @@ namespace Givaro
     public:
         typedef Givaro::Integer Integer_Type ;
         typedef Givaro::Integer Element ;
+        typedef Givaro::Integer Residu_t ;
         typedef Givaro::ZRing<Integer> Integer_Domain ;
 
-        /*! Constructor.
-         * @param bits size of integers (in bits)
-         * @param seed if \c 0 a seed will be generated, otherwise, the
-         * provided seed will be use.
-         */
-        RandomIntegerIterator(const Integer_Domain& D, size_t bits = 30, uint64_t seed = 0) :
-            _bits(bits), _integer(), _ring(D)
+    private:
+        void initialize(uint64_t seed, const size_t bits)
         {
-            if (! bits) bits = 30;
-            GIVARO_ASSERT( bits>0, "[RandomIntegerIterator] bad bit size");
+            if (! bits) _bits = 30u;
+            GIVARO_ASSERT( _bits>0u, "[RandomIntegerIterator] bad bit size");
             int64_t s=seed;
             while (!s)
                 s = static_cast<uint64_t>(BaseTimer::seed());
@@ -58,6 +56,28 @@ namespace Givaro
             this->operator++();
         }
 
+    public:
+        /*! Constructor.
+         * @param seed if \c 0 a seed will be generated, otherwise, the
+         * provided seed will be used, *** globally ***
+         * Default bit size (30) can be changed with method 'setBitsize'
+         */
+        RandomIntegerIterator(const Integer_Domain& D, uint64_t seed = 0) :
+            _bits(30u), _integer(), _ring(D)
+        {
+            initialize(seed,_bits);
+        }
+
+        /*! Constructor.
+         * @param seed if \c 0 a seed will be generated, otherwise, the
+         * provided seed will be used, *** globally ***
+         * @param samplesize is number ofpossible random values
+         */
+        RandomIntegerIterator(const Integer_Domain& D, uint64_t seed, const Integer& samplesize) : _bits(samplesize.bitsize()), _integer(), _ring(D)
+        {
+            initialize(seed,_bits);
+        }
+
         /// copy constructor.
         /// @param R random iterator to be copied.
         RandomIntegerIterator (const RandomIntegerIterator &R) :
@@ -119,7 +139,8 @@ namespace Givaro
             return this->operator()();
         }
 
-        /** @brief Sets the seed.
+        /** @brief Sets the seed *** globally ***
+         *  WARNING: i.e. sets the seed for all Givaro Integer 
          *  Set the random seed to be \p ul.
          *  @param ul the new seed.
          */
@@ -128,12 +149,13 @@ namespace Givaro
             Givaro::Integer::seeding(ul);
         }
 
-        void setBits (size_t  bits)
+        void setBitsize (size_t  bits)
         {
             _bits = bits;
+            this->operator++(); // next random value must depend on new bitsize
         }
 
-        size_t getBits () const
+        size_t getBitsize () const
         {
             return _bits ;
         }
@@ -153,9 +175,18 @@ namespace Givaro
             return Givaro::Integer::random_lessthan<_Unsigned>(a,_bits);
         }
 
-        size_t    				_bits;  	//!< common length of all integers
-        Integer_Type   			_integer;	//!< the generated integer.
-        const Integer_Domain& 	_ring;
+        size_t				_bits;	//!< common length of all integers
+        Integer_Type			_integer;	//!< the generated integer.
+        const Integer_Domain&	_ring;
+
+    private:
+        /*! DEPRECATED Constructor.
+         * This behavior can be now reproduced by calling:
+         * 1/ Constructor with 2 args, D and seed ...
+         * 2/ ... then object.setBitsize(bits).
+         */
+        RandomIntegerIterator(const Integer_Domain&, uint64_t, size_t); // DEPRECATED Constructor. Instead, call constructor with 2 args then object.setBitsize(bits)
+
 
     };
 
