@@ -16,6 +16,7 @@
 #include "givaro/givinteger.h"
 #include "givaro/givcaster.h"
 #include "givaro/givranditer.h"
+#include "givaro/givtypestring.h"
 #include "givaro/ring-interface.h"
 #include "givaro/modular-general.h"
 
@@ -51,7 +52,7 @@ namespace Givaro
 
         ModularBalanced(double p)
         : _p(p)
-          , _halfp((_p - 1.f) / 2.f)
+          , _halfp(std::floor(_p  / 2.f))
           , _mhalfp(_halfp - _p + 1.f)
           , _up(static_cast<uint32_t>(_p))
         {
@@ -75,7 +76,7 @@ namespace Givaro
         template<class T> inline T& characteristic(T& p) const { return Caster(p,_p); }
         template<class T> inline T& cardinality(T& p) const { return Caster(p,_p); }
 
-        static inline Residu_t maxCardinality() { return 134217727; } // 2^12.5
+        static inline Residu_t maxCardinality() { return 189812531; } // p=2^27.5 s.t. a*b+c fits in double, with abs(a,b,c) <= (p-1)/2
         static inline Residu_t minCardinality() { return 3.f; }
 
         // ----- Checkers
@@ -150,6 +151,11 @@ namespace Givaro
         Element& maxpy  (Element& r, const Element& a, const Element& x, const Element& y) const;
         Element& maxpyin(Element& r, const Element& a, const Element& x) const;
 
+        // -- type_string
+        static const std::string type_string () {
+            return "ModularBalanced<" + TypeString<Element>::get() +  ">";
+        }
+
         // ----- Random generators
         typedef ModularRandIter<Self_t> RandIter;
         typedef GeneralRingNonZeroRandIter<Self_t> NonZeroRandIter;
