@@ -10,9 +10,12 @@
 #ifndef __GIVARO_MODULAR_EXTENDED_H
 #define __GIVARO_MODULAR_EXTENDED_H
 
+#include <type_traits>
+
 #include "givaro/givconfig.h"
 
 #include "givaro/givranditer.h"
+#include "givaro/givtypestring.h"
 #include "givaro/ring-interface.h"
 #include "givaro/modular-general.h"
 
@@ -38,6 +41,11 @@ namespace Givaro{
         typedef uint64_t Residu_t;
         enum { size_rep = sizeof(Residu_t) };
 
+        using is_elt_integral = std::false_type;
+        static constexpr bool is_elt_integral_v = false;
+        using is_elt_floating_point = std::true_type;
+        static constexpr bool is_elt_floating_point_v = true;
+
     private:
         // Verkampt Split
         inline void split(const Element x, Element &x_h, Element &x_l) const {
@@ -53,6 +61,10 @@ namespace Givaro{
         }
 
         // Dekker mult, a * b = s + t
+        /* Note: this function may not work correctly when compiled for the 387
+         * floating point coprocessor which may store temporary on 80-bit
+         * registry.
+         */
         inline void mult_dekker(const Element a, const Element b, Element &s, Element &t) const{
             s = a*b;
             Element ah, al, bh, bl;
@@ -232,6 +244,11 @@ namespace Givaro{
             return maxpy(r, a, x, r);
         }
 
+        // -- type_string
+        static const std::string type_string () {
+            return "ModularExtended<" + TypeString<Element>::get() +  ">";
+        }
+
         // ----- Random generators
         typedef ModularRandIter<Self_t> RandIter;
         typedef GeneralRingNonZeroRandIter<Self_t> NonZeroRandIter;
