@@ -16,6 +16,7 @@
 #include "givaro/givinteger.h"
 #include "givaro/givcaster.h"
 #include "givaro/givranditer.h"
+#include "givaro/givtypestring.h"
 #include "givaro/modular-general.h"
 #include "givaro/ring-interface.h"
 #include "givaro/modular-implem.h"
@@ -90,6 +91,11 @@ namespace Givaro
         Element& maxpy  (Element& r, const Element& a, const Element& x, const Element& y) const;
         Element& maxpyin(Element& r, const Element& a, const Element& x) const;
 
+        // -- type_string
+        static const std::string type_string () {
+            return "Modular<" + TypeString<Integer>::get() +  ">";
+        }
+
         // ----- Random generators
         typedef ModularRandIter<Self_t> RandIter;
         typedef GeneralRingNonZeroRandIter<Self_t> NonZeroRandIter;
@@ -103,16 +109,17 @@ namespace Givaro
     };
 
 
-    /* Specialisation for Modular<integer> field*/
+    /* Specialisation for Modular<integer> ring*/
     template <>
-    class ModularRandIter<Modular<Integer, Integer> >
+    class ModularRandIter<Modular<Integer> >
     {
     public:
         typedef Modular<Integer>  Ring;
-        typedef Ring::Element Element;
+        typedef Integer Element;
+        typedef Integer Residu_t;
 
-        ModularRandIter(const Ring& R, const size_t& size = 0, const size_t& seed = 0)
-        : _ring(R)
+        ModularRandIter(const Ring& R, const size_t& seed = 0, const Integer size = 0)
+        : _size(size?size:R.cardinality()), _ring(R)
         {
             // GivRandom will select a non-zero value, even if seed is 0
             GivRandom generator(seed);
@@ -121,9 +128,9 @@ namespace Givaro
         Element& operator()(Element& elt)
         {
             // Create new random Elements
-            Givaro::Integer::random_lessthan(elt,_ring.residu());
-
-            return elt;
+	  Element tmp;
+            Givaro::Integer::random_lessthan(tmp,_size);
+	    return _ring.init(elt,tmp);
         }
 
         Element& random(Element& elt)
@@ -143,6 +150,7 @@ namespace Givaro
         const Ring& ring() const { return _ring; }
 
     private:
+        const Residu_t _size;
         const Ring& _ring;
 
     }; //  class ModularRandIter<Integer>
